@@ -1,6 +1,7 @@
 """Starlette integration"""
 
 import contextvars
+import typing
 
 import pydantic
 import starlette.requests
@@ -9,6 +10,10 @@ import starlette.middleware.base
 from .model import BaseReferrableModel
 
 
+_URL_MODEL: typing.Type[pydantic.BaseModel] = pydantic.create_model(
+    "_URL_MODEL", __root__=(pydantic.AnyHttpUrl, ...)
+)
+
 _request_var: contextvars.ContextVar[
     starlette.requests.Request
 ] = contextvars.ContextVar("_request_var")
@@ -58,41 +63,31 @@ class ReferrableModel(BaseReferrableModel):
     please refer to :ref:`quickstart`.
     """
 
-    @staticmethod
-    def get_url_type():
-        return pydantic.AnyHttpUrl
+    @classmethod
+    def parse_as_url(cls, value: typing.Any) -> typing.Optional[pydantic.AnyHttpUrl]:
+        """Parse ``value`` as ``pydantic.AnyHttpUrl``"""
+        return cls.try_parse_as(_URL_MODEL, value)
 
     @classmethod
-    def key_to_url(cls, key):
+    def key_to_url(cls, key) -> pydantic.AnyHttpUrl:
         request = _request_var.get()
         details_view = cls._get_details_view()
-        key_names = cls._key_names
-        if len(key_names) > 1:
-            kwargs = dict(zip(key_names, key))
-        else:
-            kwargs = {key_names[0]: key}
+        kwargs = cls.key_to_path_params(key)
         return pydantic.parse_obj_as(
             pydantic.AnyHttpUrl, request.url_for(details_view, **kwargs)
         )
 
     @classmethod
-    def url_to_key(cls, url: pydantic.AnyHttpUrl):
+    def url_to_key(cls, url: pydantic.AnyHttpUrl) -> typing.Any:
         request = _request_var.get()
         details_view = cls._get_details_view()
-        key_names = cls._key_names
         for route in request.app.routes:
             if route.name == details_view:
                 _, scope = route.matches(
                     {"type": "http", "method": "GET", "path": url.path}
                 )
                 if scope:
-                    if len(key_names) > 1:
-                        path_params = scope["path_params"]
-                        key = [path_params[name] for name in key_names]
-                    else:
-                        key_name = key_names[0]
-                        key = scope["path_params"][key_name]
-                    return pydantic.parse_obj_as(cls.get_key_type(), key)
+                    return cls.path_params_to_key(scope["path_params"])
         raise ValueError(f"Could not resolve {url} into key")
 
     @classmethod
