@@ -1,6 +1,7 @@
 import json
 import typing
 
+import packaging.version as version
 from hypothesis import given, strategies as st
 import pydantic
 import pytest
@@ -10,6 +11,13 @@ from hrefs import Href, BaseReferrableModel, PrimaryKey
 from util import hrefs
 
 
+def _pydantic_does_not_support_field_in_modify_schema():
+    try:
+        return version.parse(pydantic.__version__) < version.Version("1.9")
+    except AttributeError:
+        return True
+
+
 class Pet(BaseReferrableModel):
     id: int
 
@@ -123,3 +131,29 @@ def test_href_forward_reference():
     MyModel.update_forward_refs()
 
     assert MyModel(id=1).self == Href(key=1, url="/1")
+
+
+@given(st.integers(), st.floats())
+def test_derived_model_inherits_referrable_properties(key, purr_frequency):
+    class Cat(Pet):
+        purr_frequency: float
+
+    cat = Cat(id=key, purr_frequency=purr_frequency)
+    href = pydantic.parse_obj_as(Href[Cat], cat)
+    assert href.key == key
+    assert href.url == Pet.key_to_url(key)
+
+
+@pytest.mark.skipif(
+    _pydantic_does_not_support_field_in_modify_schema(),
+    reason="pydantic does not support field argument in __modify_schema__",
+)
+def test_href_schema():
+    owner_schema = Owner.schema()
+    assert owner_schema["properties"]["pets"] == {
+        "title": "Pets",
+        "type": "array",
+        "items": {
+            "type": "string",
+        },
+    }
